#include <stdio.h>
#include <fcntl.h>
#include <malloc.h>
#include <string.h>
#include <strings.h>
#include <X11/Xlib.h>
#include "sherlock.h"
#include "error.h"

extern FILE *yyin;

/*Start up interpreting for batch mode operation.

  Inputs:  fp   - file pointer to file containing commands to interpret (set
		  to stdin for interactive batch mode)
  Outputs: none
  Locals:  none
  Globals: yyin - file pointer that lex uses to read in data
*/
batchmode(fp)
register FILE *fp;
{
	yyin = fp;

	prompt();
	yyparse();
}

/*Run a pattern matching algorithm (in batch mode).

  Inputs:  alg - algorithm to run
  Outputs: none
  Locals:  none
  Globals: none
*/
runalg(alg)
register int alg;
{
	run(alg);

	/*Sound a bell to let the user know Sherlock is done*/
	puts("");

	/*Display the results of the algorithm*/
	display_results();
	puts("");
}

/*Load an image or pattern file into memory (in batch mode).

  Inputs:  input    - file to read in
	   type     - specifies where it's an image or pattern file
  Outputs: none
  Locals:  err      - error from readheader (if there is one)
	   fd       - file descriptor of file to read in
  Globals: errno    - number of last error encountered
	   filename - file read in
	   FATAL    - error is fatal, program will terminate
	   NULL     - 0
	   O_RDONLY - open the file for reading only
	   WARNING  - error message isn't fatal, continue execution
*/
loadfile(type,input)
register int type;
register char *input;
{
	register int fd,err;

	/*Store filename*/
	if(filename[type]) free(filename[type]);
	if((filename[type] = strdup(input)) == NULL)
	  error(FATAL,errno);

	if((fd = open(filename[type],O_RDONLY)) == -1) error(WARNING,errno);
	else if(!(err = readheader(fd,type))) {
	       /*Read the image into memory*/
	       readimage(fd,type,NULL,(HEADER *) NULL);

	       /*Set up location within image, effectively, it doesn't
		 come from the in memory image, so it's values here
		 are meaningless*/
	       getinfo(0,0,-1,-1);
	     } else error(WARNING,err);
}

/*Save a pattern from memory to disk (in batch mode).

  Inputs:  output     - file to save data to
  Outputs: none
  Locals:  err        - error from readheader (if there is one)
	   fd         - file descriptor of file to read in
  Globals: data       - un(color)mapped image data
	   errno      - number of last error encountered
	   ENOPATTERN - "no pattern in memory to save" error
	   O_CREAT    - create the file if it doesn't exist
	   O_TRUNC    - truncate the file if it already exists
	   O_WRONLY   - open the file for writing only
	   PATTERN    - reference the pattern element of an array
	   WARNING    - error message isn't fatal, continue execution
*/
savefile(output)
register char *output;
{
	register int fd,err;

	/*Store filename*/
	if(filename[PATTERN]) free(filename[PATTERN]);
	if((filename[PATTERN] = strdup(output)) == NULL)
	  error(FATAL,errno);

	/*Save data if there is data to save*/
	if(data[PATTERN]) {
	  /*Create a file with the mode of 644 (octal)*/
	  if((fd = open(filename[PATTERN],O_WRONLY | O_CREAT | O_TRUNC,0644)) == -1)
	    error(WARNING,errno);
	  else if(!(err = writeheader(fd))) writeimage(fd);
	       else error(WARNING,err);
	} else error(WARNING,ENOPATTERN);
}

/*Crop out a piece from the image (in batch mode) to make a pattern.

  Inputs:  height - height of area to crop out
	   width  - width of area to crop out
	   x      - starting X coordinate
	   y      - starting Y coordinate
  Outputs: none 
  Locals:  area   - cropped out area
  Globals: IMAGE  - reference the image element of an array
*/
crop(x,y,width,height)
register long x,y,width,height;
{
	register XRectangle area;

	area.x = x;
	area.y = y;
	area.width = width;
	area.height = height;

	/*Set up a mini header that tells where to find the pattern within
	  the image data, and then get the data*/
	getinfo(x,y,width,height);
	getimage(IMAGE,area);
}

/*Run the shell escape command.

  Inputs:  string    - command to run
  Outputs: none
  Locals:  argv      - argument list to execute
	   loop      - loop through words in string
	   numspaces - number of spaces in command string
	   pointer   - pointer to words in command string
	   space     - pointer to space between words
  Globals: errno     - number of last error encountered
	   FATAL     - error is fatal, program will terminate
	   NULL      - 0
*/
shell_escape(string)
register char *string;
{
	register int numspaces,loop;
	register char **argv,*pointer,*space;

	numspaces = 0;
	pointer = string;

	/*Find out how many spaces are in the command string and this
	  will tell us how many words there are*/
	while(pointer = index(pointer + 1,' ')) ++numspaces;

	/*Allocate space for the words (which is the number of spaces + 1)
	  plus another for the ending NULL*/
	if((argv = (char **) calloc(numspaces + 2,sizeof(char *))) == NULL)
	  error(FATAL,errno);

	/*Create argument list*/
	space = index(string + 1,' ');
	for(loop = 0,pointer = string;space;++loop) {
	  *space = NULL;

	  if((argv[loop] = strdup(pointer)) == NULL) error(FATAL,errno);

	  pointer = space + 1;

	  *space = ' ';

	  space = index(pointer + 1,' ');
	}
	if((argv[loop++] = strdup(pointer)) == NULL) error(FATAL,errno);
	argv[loop] = NULL;

	/*Execute*/
	switch(fork()) {
	  case 0:				/*Child*/
			execvp(argv[0],argv);
	  case -1:				/*Error*/
			error(FATAL,errno);
	  default:				/*Parent*/
			wait(NULL);		/*Wait for child to finish*/
	}
}

