#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include "ui_globals.h"
#include "sherlock.h"
#include "xvgr.h"
#include "error.h"

extern FILE *corr_resfp;

/*Parse command line arguments.  Return a file pointer to a file containing
  batch commands if the "-b" option was given.

  Inputs:  argc         - number of command line arguments
	   argv         - command line arguments
	   fd           - file descriptor of image/pattern data files
  Outputs: batchfp      - file pointer to file containing batch commands
  Locals:  batchfile    - name of file containing batch commands
	   batchfp      - file pointer to file containing batch commands
	   corr_results - name of file to save correlation data to
	   loop         - loop through arguments
	   pointer      - point to option letter
  Globals: batchflag    - batch mode flag
	   corr_resfp   - file pointer to correlation graph data
	   errno        - number of last error encountered
	   filename     - name of files containing image/pattern data
	   info	        - structure containing results of the algorithm
           invert_bias  - flag indicating to bias invert operation
	   stdin        - standard input
	   subtitle     - subtitle for graph
	   title        - title for graph
	   AUTO_INVERT  - automatically determine inversion
	   IMAGE        - reference the image element of an array
	   PATTERN      - reference to the pattern element of an array
*/
FILE *getargs(argc,argv,fd)
register int argc,fd[];
register char *argv[];
{
	register int loop;
	register char *pointer,*batchfile,*corr_results;
	register FILE *batchfp = NULL;

	/*Set initial conditions*/
	batchflag = 0;
	invert_bias[0] = invert_bias[1] = AUTO_INVERT;
	fd[PATTERN] = -1;
	batchfile = NULL;
	title = subtitle = NULL;
	corr_results = DEF_CORR_RESULTS_FILE;
	dist_results = DEF_DIST_RESULTS_FILE;

	/*Loop through arguments*/
	for(loop = 1;loop < argc;++loop) {
	  pointer = argv[loop];
	  if(*pointer == '-') {
	    ++pointer;

	    switch(*pointer) {
	      case 'b':			/*-b batch mode*/
		if(++loop >= argc) error(FATAL,EUSAGE);
		else batchfile = argv[loop];

		break;
	      case 'i':
		if(++loop >= argc) error(FATAL,EUSAGE);
		else switch(*(pointer + 1)) {
		       case 'i':	/*-ii { y | n }*/
				if(!strcmp(argv[loop],"y"))
				  invert_bias[IMAGE] = YES_INVERT;
				else if(!strcmp(argv[loop],"n"))
				       invert_bias[IMAGE] = NO_INVERT;
				     else error(FATAL,EUSAGE);
				break;
		       case 'p':	/*-ip { y | n }*/
				if(!strcmp(argv[loop],"y"))
				  invert_bias[PATTERN] = YES_INVERT;
				else if(!strcmp(argv[loop],"n"))
				       invert_bias[PATTERN] = NO_INVERT;
				     else error(FATAL,EUSAGE);
				break;
		       case '\0':	/*-i filename*/
				if((filename[IMAGE] = strdup(argv[loop])) ==
				   NULL)
				  error(FATAL,errno);
				break;
		       default:
				error(FATAL,EUSAGE);
		     }

		break;
	      case 'l':			/*-l 'p1 p2 xinc yinc'*/
		if(++loop >= argc) error(FATAL,EUSAGE);
		else if(sscanf(argv[loop],"%d %d %d %d",&info.p1,&info.p2,
			&info.k1,&info.k2) != 4)
		       error(FATAL,EUSAGE);

		break;
	      case 'p':			/*-p filename*/
		if(++loop >= argc) error(FATAL,EUSAGE);
		else if((filename[PATTERN] = strdup(argv[loop])) == NULL)
		       error(FATAL,errno);

		break;
	      case 'r':
		if(++loop >= argc) error(FATAL,EUSAGE);
		else switch(*(pointer + 1)) {
		       case 'c':	/*-rc correlation graph data*/
				corr_results = argv[loop];

				break;
		       case 'd':	/*-rd distance graph data*/
				if((dist_results = strdup(argv[loop])) == NULL)
				  error(FATAL,errno);

				break;
		       case 's':	/*-rs number*/
				info.seed = atol(argv[loop]);

				break;
		       default:
				error(FATAL,EUSAGE);
		     }

		break;
	      case 's':			/*-s subtitle string*/
		if(++loop >= argc) error(FATAL,EUSAGE);
		else if((subtitle = strdup(argv[loop])) == NULL)
		       error(FATAL,errno);

		break;
	      case 't':			/*-t title string*/
		if(++loop >= argc) error(FATAL,EUSAGE);
		else if((title = strdup(argv[loop])) == NULL)
		       error(FATAL,errno);

		break;
	      case 'v':			/*-v show version and quit*/
		error(FATAL,SHOW_VERSION);
	      default:
		error(FATAL,EUSAGE);
	    }
	  }
	}

	/*It's an error if the "-i" option isn't given*/
	if(!filename[IMAGE]) error(FATAL,ENOIMAGE);
	else if((fd[IMAGE] = open(filename[IMAGE],O_RDONLY)) == -1)
	       error(FATAL,errno);

	/*Read in pattern data if a filename is given*/
	if(filename[PATTERN])
	  if((fd[PATTERN] = open(filename[PATTERN],O_RDONLY)) == -1)
	    error(FATAL,errno);
	  else getinfo(0,0,-1,-1);

	/*Set up batch mode if given*/
	if(batchfile) {
	  setbit(BATCHMODE);

	  if(!strcmp(batchfile,"-")) batchfp = stdin;
	  else if((batchfp = fopen(batchfile,"r")) == NULL) error(FATAL,errno);
		 else setbit(BATCHFILE);
	}

	/*Write viewport commands to graph file*/
	if((corr_resfp = fopen(corr_results,"w")) == NULL)
	  error(FATAL,errno);
	set_y_viewport(corr_resfp);

	/*Start child timer process*/
	init_timer();

	return(batchfp);
}

/*Get information about where the pattern is within the image data.

  Inputs:  pixels       - number of pixels in the pattern
	   scanlines    - number of scanlines in the pattern
	   xoffset      - X coordinate offset into image data
	   yoffset      - Y coordinate offset into image data
  Outputs: none
  Locals:  none
  Globals: pattern_info - pattern location information
*/
getinfo(xoffset,yoffset,pixels,scanlines)
register long xoffset,yoffset,pixels,scanlines;
{
	pattern_info.xoffset = xoffset;
	pattern_info.yoffset = yoffset;
	pattern_info.pixels = pixels;
	pattern_info.scanlines = scanlines;
}

/*Crop out an area from image data.

  Inputs:  area       - area coordinates to crop
	   whichimage - which data set to crop (image or pattern)
  Outputs: none
  Locals:  oldheader  - old header information
  Globals: header     - image/pattern header information arrays
	   BATCHMODE  - flag bit indicating Sherlock is running in batch mode
	   NULL       - 0
	   PATTERN    - reference the pattern element of an array
*/
getimage(whichimage,area)
register int whichimage;
register XRectangle area;
{
	HEADER oldheader;

	/*Indicate to the user what's happening*/
	if(isbitclear(BATCHMODE))
	  set_footer("Retrieving pattern...",whichimage);

	oldheader = header[whichimage];

	/*Create new header*/
	header[PATTERN].pixels = area.width;
	header[PATTERN].scanlines = area.height;
	header[PATTERN].xcenter = area.x + (area.width / 2);
	header[PATTERN].ycenter = area.y + (area.height / 2);

	/*Resize pattern window*/
	if(isbitclear(BATCHMODE))
	  resize(PATTERN);

	/*Crop data to become the new pattern*/
	readimage(NULL,PATTERN,whichimage,&oldheader);

	/*If not in batch mode, redraw new pattern and indicate that
	  Sherlock is ready to continue*/
	if(isbitclear(BATCHMODE)) {
	  repaint(canvas[PATTERN]);

	  set_footer("Ready",whichimage);
	}
}

