#include <fcntl.h>
#include "ui.h"
#include "ui_globals.h"
#include "sherlock.h"
#include "error.h"

/*Read the header from a Sherlock image file and determine if it's a valid
  image file.  If it's a pattern header, it must be equal or less than the
  size of the image.

  Inputs:  fd           - file descriptor of file to read
	   which        - which file is being read (image or pattern)
  Outputs: 0 if no error, non-zero otherwise with the value indicating the
	   error
  Locals:  magic        - magic number
  Globals: errno        - number of last error encountered
	   header       - image/pattern header information arrays
	   EBADFORMAT   - "image in unknown format" error
	   ETOOBIG      - "pattern too big for given image" error
	   IMAGE        - reference the image element of an array
	   MAGIC_NUMBER - Sherlock image format magic number
	   PATTERN      - reference the pattern element of an array
*/
readheader(fd,which)
register int fd,which;
{
	long magic;

	/*Read magic number and see if it's correct*/
	if(read(fd,(char *) &magic,sizeof(magic)) == -1) return(errno);
	if(magic != MAGIC_NUMBER) return(EBADFORMAT);

	/*Read header information*/
	if(read(fd,(char *) &header[which],sizeof(header[which])) == -1)
	  return(errno);

	/*Make sure pattern isn't bigger than image*/
	if(which == PATTERN)
	  if((header[PATTERN].pixels > header[IMAGE].pixels) ||
	     (header[PATTERN].scanlines > header[IMAGE].scanlines))
	    return(ETOOBIG);

	return(0);
}

/*Write a Sherlock image header to a file.

  Inputs:  fd           - file descriptor to write to
  Outputs: 0 if no error, non-zero otherwise with the value indicating the
	   error
  Locals:  magic        - magic number
  Globals: errno        - number of last error encountered
	   header       - image/pattern header information arrays
	   MAGIC_NUMBER - Sherlock image format magic number
	   PATTERN      - reference the pattern element of an array
*/
writeheader(fd)
register int fd;
{
	long magic;

	magic = MAGIC_NUMBER;

	/*Write magic number*/
	if(write(fd,(char *) &magic,sizeof(magic)) == -1)
	  return(errno);

	/*Write header*/
	if(write(fd,(char *) &header[PATTERN],sizeof(header[PATTERN])) == -1)
	  return(errno);

	return(0);
}

/*Read in image/pattern data passing it through the colormap lookup table.

  Inputs:  fd           - file descriptor to read from
	   from         - non-zero indicates to get data from data in memory
			  rather that from a file
	   fromheader   - header of data to to get this data from (in memory)
	   which        - which data is being read in (image or pattern)
  Outputs: none
  Locals:  loop         - loop through pixels and scanlines
	   mapped_data  - data after going through colormap lookup table
	   tmp          - pointer to data in memory
	   tmp2	        - pointer to data in memory
  Globals: cmlut        - colormap lookup table
	   data         - un(color)mapped image data
	   dpy          - X display
	   errno        - number of last error encountered
	   header       - image/pattern header information arrays
	   image        - array containing image and pattern data
	   pattern_info - pattern location information
	   BATCHMODE    - flag bit indicating Sherlock is running in batch mode
	   FATAL        - error is fatal, program will terminate
	   LOADIMAGE    - which action is to be performed (load an image)
	   NULL         - 0
	   StaticGray   - display static gray image
	   ZPixmap      - data is coming in XY
*/
readimage(fd,which,from,fromheader)
register int fd,which,from;
register HEADER *fromheader;
{
	register long loop;
	register unsigned char *tmp = NULL,*tmp2;
	static unsigned char *mapped_data[2];

	if(isbitclear(BATCHMODE)) {
	  /*Let the user know what's happening*/
	  if(which == LOADIMAGE) set_footer("Loading image...",which);
	  else set_footer("Loading pattern...",which);

	  /*Destroy previous image (if there is one)*/
	  if(image[which]) {
	    XDestroyImage(image[which]);
	    tmp = data[which];
	  } else tmp = NULL;
	}

	if((data[which] = (unsigned char *) malloc(header[which].pixels *
					    header[which].scanlines)) == NULL)
	  error(FATAL,errno);
	if(isbitclear(BATCHMODE))
	  if((mapped_data[which] = (unsigned char *) malloc(header[which].pixels
					    * header[which].scanlines)) == NULL)
	    error(FATAL,errno);

	/*Set up pointers to data in memory if getting it from there*/
	if(fd) tmp2 = tmp;
	else tmp2 = data[from];

	/*Loop through scanlines reading in data*/
	for(loop = 0;loop < header[which].scanlines;++loop)
	  if(fd) {
	    if(read(fd,(char *) &data[which][loop * header[which].pixels],
	            header[which].pixels) == -1)
	      error(FATAL,errno);
	  } else bcopy(&tmp2[fromheader->pixels *
		       (loop + pattern_info.yoffset) + pattern_info.xoffset],
		       &data[which][loop * header[which].pixels],
		       header[which].pixels);

	if(fd) close(fd);
	if(tmp) free(tmp);

	if(isbitclear(BATCHMODE)) {
	  /*Pass data through colormap lookup table so that it displays
	    with the correct grayscale colormap*/
	  for(loop = 0;loop < header[which].pixels * header[which].scanlines;
	      ++loop)
	    mapped_data[which][loop] = (unsigned char) cmlut[data[which][loop]];

	  image[which] = XCreateImage(dpy,StaticGray,8,ZPixmap,0,
				      mapped_data[which],header[which].pixels,
				      header[which].scanlines,8,
			              header[which].pixels);
	}
}

/*Write out image/pattern data to a file.  Real unmapped data is written out,
  not mapped data.  Only the pattern can be written out (no need to write
  the image).

  Inputs:  fd        - file descriptor to write to
  Outputs: none
  Locals:  loop      - loop through scanlines
  Globals: data      - un(color)mapped image data
	   errno     - number of last error encountered
	   header    - image/pattern header information arrays
	   BATCHMODE - flag bit indicating Sherlock is running in batch mode
	   FATAL     - error is fatal, program will terminate
	   PATTERN   - reference the pattern element of an array
*/
writeimage(fd)
register int fd;
{
	register long loop;

	/*Let user know what's happening*/
	if(isbitclear(BATCHMODE))
	  set_footer("Saving pattern...",PATTERN);

	/*Write out scanlines*/
	for(loop = 0;loop < header[PATTERN].scanlines;++loop) {
	  if(write(fd,(char *) &data[PATTERN][loop * header[PATTERN].pixels],
	           header[PATTERN].pixels) == -1)
	    error(FATAL,errno);
	}

	close(fd);

	/*Sherlock is ready to continue*/
	if(isbitclear(BATCHMODE))
	  set_footer("Ready",PATTERN);
}

