#include <stdio.h>
#include <math.h>
#include <values.h>
#include "sherlock.h"
#include "error.h"

/*Calculate a space domain correlation using Random Ordering.  The
  steps are as follows:

  1) randomize pixel locations
  2) go through randomized list, finding the minimum correlation
     at each point

  Because of the way the correlation is calculated, (it's actually a
  difference rather than a correlation) we look for the smallest
  value, rather than the largest.

  Inputs:  none
  Outputs: none
  Locals:  loop      - loop until all pixels in image covered
	   min       - minimum correlation value
	   randnums  - list of randimized positions
	   res       - result of correlation
	   total     - total number of pixels in image
	   xcoord    - X coordinate to correlate at
	   ycoord    - Y coordinate to correlate at
  Globals: header    - image/pattern header information arrays
	   BATCHMODE - flag bit indicating Sherlock is running in batch mode
	   IMAGE     - reference the image element of an array
	   MAXDOUBLE - maximum value of a double variable
*/
rand_alg()
{
	register long loop,xcoord,ycoord,total;
	long *randnums;
	register double min,res;

	/*Set initial conditions*/
	min = MAXDOUBLE;
	total = header[IMAGE].pixels * header[IMAGE].scanlines;
	randomize(&randnums);

	/*Loop through pixels*/
	for(loop = 0;loop < total;++loop) {
	  if(isbitclear(BATCHMODE) && !(loop % header[IMAGE].pixels))
	    update_gauge((double) loop / (double) total);

	  xcoord = randnums[loop] % header[IMAGE].pixels;
	  ycoord = randnums[loop] / header[IMAGE].pixels;

	  /*Calculate correlation and compare for minimum*/
	  if((res = corr(xcoord,ycoord,loop)) < min) {
	    min = res;
	    info.x = xcoord;
	    info.y = ycoord;
	  }
	}

	free((char *) randnums);
}

/*Randomize a list of numbers.

  Inputs:  none
  Outputs: array     - list of randomized numbers
  Locals:  checklist - checklist to make sure we cover each number in list
	   good      - indicates random choice was good
	   loop      - loop through numbers to be randomized
	   num       - random number
	   total     - total number of numbers to randomize
  Globals: errno     - number of last error encountered
	   header    - image/pattern header information arrays
	   info      - structure containing results of the algorithm
	   BATCHMODE - flag bit indicating Sherlock is running in batch mode
	   FATAL     - error is fatal, program will terminate
	   IMAGE     - reference the image element of an array
	   NULL      - 0
*/
randomize(array)
register long **array;
{
	register int good;
	register long loop,num,*checklist,total;

	/*Let the user know what's happening*/
	if(isbitclear(BATCHMODE))
	  set_footer("Randomizing...",IMAGE);

	/*Set up to randomize*/
	total = header[IMAGE].pixels * header[IMAGE].scanlines;
	if(info.seed == -1) info.seed = (long) time(NULL);
	srand48(info.seed);

	if((*array = (long *) calloc(total,sizeof(long))) == NULL)
	  error(FATAL,errno);
	if((checklist = (long *) calloc(total,sizeof(long))) == NULL)
	  error(FATAL,errno);

	/*Go through numbers*/
	for(loop = 0,good = 0;loop < total;++loop,good = 0)
	  while(!good) {
	    num = lrand48() % total;

	    if(!checklist[num]) {
	      (*array)[loop] = num;
	      checklist[num] = 1;
	      good = 1;
	    }
	  }

	free((char *) checklist);

	/*Keep user informed*/
	if(isbitclear(BATCHMODE))
	  set_footer("Searching for pattern...",IMAGE);
}

