#include <math.h>
#include "ui.h"
#include "ui_globals.h"
#include "sherlock.h"
#include "timing/timing.h"

GC norm_gc,xor_gc;

/*Set the left footer string of a window (used to indicate status).

  Inputs:  footer            - string to display
	   which             - which window to display in
  Outputs: none
  Locals:  none
  Globals: frame             - UI window handles
	   FRAME_LEFT_FOOTER - window left footer attribute
	   NULL              - 0
	   TRUE              - turn an attribute on
	   XV_SHOW           - map an object to the display
*/
set_footer(footer,which)
register int which;
register char *footer;
{
	xv_set(frame[which],FRAME_LEFT_FOOTER,footer,XV_SHOW,TRUE,NULL);
}

/*Display an image in a canvas.  This is called whenever a canvas has a
  repaint (refresh) event.

  Inputs:  exposed_canvas  - canvas that needs repainting
  Outputs: none
  Locals:  which           - which canvas need repainting (image or pattern)
  Globals: canvas          - UI canvas handles (that display the images)
	   dpy		   - X display
	   header	   - image/pattern header information arrays
	   image	   - array containing image and pattern data
	   norm_gc	   - normal graphics context
	   IMAGE	   - reference the image element of an array
	   PATTERN	   - reference the pattern element of an array
	   WIN_CLIENT_DATA - data attached to an xview object
*/
void repaint(exposed_canvas)
register Canvas exposed_canvas;
{
	register int which;

	/*Find out which canvas needs repainting*/
	which = (exposed_canvas == canvas[IMAGE])?IMAGE:PATTERN;

	if(image[which]) {
	  set_footer("Displaying image...",which);

	  /*Put data into canvas*/
	  XPutImage(dpy,xv_get(canvas[which],WIN_CLIENT_DATA),norm_gc,
		    image[which],0,0,0,0,header[which].pixels,
		    header[which].scanlines);

	  set_footer("Ready",which);
	}
}

/*Resize the dimensions of a window.

  Inputs:  which        - which window needs to be resized
  Outputs: none
  Locals:  height       - new height of the window
  Globals: canvas       - UI canvas handles (that display the images)
	   frame        - UI window handles
	   header       - image/pattern header information arrays
	   IMAGE        - reference the image element of an array
	   NULL         - 0
	   PANEL_HEIGHT - height of control panel
	   TRUE         - turn an attribute on
	   XV_HEIGHT    - window height attribute
	   XV_SHOW      - map a window to the display (i.e. display window)
	   XV_WIDTH     - window width attribute
*/
resize(which)
register int which;
{
	register int height;

	set_footer("Resizing...",which);

	/*Figure out the height, if it's for the base frame, add on the
	  height of the control panel (where the menu buttons are)*/
	if(which == IMAGE) height = PANEL_HEIGHT + header[which].scanlines;
	else height = header[which].scanlines;

	xv_set(frame[which],XV_WIDTH,header[which].pixels,XV_HEIGHT,height,
	       XV_SHOW,TRUE,NULL);

	/*Resize canvas to fit new window size*/
	window_fit(canvas[which]);
}

/*Activate a menu button or an individual menu choice.

  Inputs:  handle            - handle to menu button containing object to
			       activate
	   item              - menu item to activate
  Outputs: none
  Locals:  none
  Globals: panel             - handle to command panel
	   FALSE             - turn attribute off
	   MENU_INACTIVE     - grayed out attribute (for menu choices)
	   MENU_NTH_ITEM     - Nth menu choice in a menu button
	   NULL		     - 0
	   PANEL_INACTIVE    - grayed out attribute (for panel objects)
	   RUN_BUTTON_HANDLE - handle to UI "Run" menu button
	   XV_KEY_DATA       - data attached to an xview object

*/
activate(handle,item)
register int handle,item;
{
	if(handle == RUN_BUTTON_HANDLE)
	  xv_set(xv_get(panel,XV_KEY_DATA,handle),PANEL_INACTIVE,FALSE,NULL);
	else xv_set(xv_get(xv_get(panel,XV_KEY_DATA,handle),MENU_NTH_ITEM,item),
	       	    MENU_INACTIVE,FALSE,NULL);
}

/*Deactivate a menu button or an individual menu choice.

  Inputs:  handle            - handle to menu button containing object to
                               activate
           item              - menu item to activate
  Outputs: none
  Locals:  none
  Globals: panel             - handle to command panel
           MENU_INACTIVE     - grayed out attribute (for menu choices)
           MENU_NTH_ITEM     - Nth menu choice in a menu button
           NULL              - 0 
           PANEL_INACTIVE    - grayed out attribute (for panel objects)
           RUN_BUTTON_HANDLE - handle to UI "Run" menu button
	   TRUE		     - turn attribute on
           XV_KEY_DATA       - data attached to an xview object

*/
deactivate(handle,item)
register int handle,item;
{
	if(handle == RUN_BUTTON_HANDLE)
	  xv_set(xv_get(panel,XV_KEY_DATA,handle),PANEL_INACTIVE,TRUE,NULL);
	else xv_set(xv_get(xv_get(panel,XV_KEY_DATA,handle),MENU_NTH_ITEM,item),
	       	    MENU_INACTIVE,TRUE,NULL);
}

/*Draw a rectangle.

  Inputs:  drawable - object to draw box in
	   rect     - rectangle dimensions
  Outputs: none
  Locals:  none
  Globals: dpy      - X display
	   xor_gc   - exclusive or graphics context to draw cropping box with
*/
drawbox(drawable,rect)
register Drawable drawable;
register XRectangle rect;
{
	XDrawRectangle(dpy,drawable,xor_gc,rect.x,rect.y,rect.width,
		       rect.height);
}

/*Update the gauge object to show progression.

  Inputs:  percentage   - percentage (from 0.0 to 1.0) of completion
  Outputs: none
  Locals:  none
  Globals: dpy          - X display
	   frame        - UI window handles
	   GAUGE        - reference the xview gauge object of an array
	   GAUGE_HANDLE - handle to UI gauge panel item
	   NULL         - 0
	   PANEL_VALUE  - value of UI panel item
	   XV_KEY_DATA  - data attached to an xview object
*/
update_gauge(percentage)
register double percentage;
{
	xv_set(xv_get(frame[GAUGE],XV_KEY_DATA,GAUGE_HANDLE),PANEL_VALUE,
	       nint(percentage * 100.0),NULL);

	XFlush(dpy);
}

/*Display the results of an algorithm running.  If running in batch mode,
  the results are sent to the screen.  In UI mode, results are sent to
  their own window.  For each algorithm, the algorithm that ran, ratio
  between image and pattern, time spent, and pattern position are given.
  In the case of LPS, the maximum numbers used are given.  In the case
  of Random Ordering, the random seed is given.

  Inputs:  none
  Outputs: none
  Locals:  gc           - graphics context used to render the text
	   string       - text to display
	   xwin         - window to write text in
  Globals: dpy          - X display
	   filename     - image/pattern filenames
	   frame        - UI window handles
	   header       - image/pattern header information arrays
	   info         - structure containing results of the algorithm
	   BATCHMODE    - flag bit indicating Sherlock is running in batch mode
	   FREQ_DOMAIN  - algorithm that ran was frequency domain correlation
	   GC_HANDLE    - handle to graphics context
	   LPS          - algorithm that ran was linear pixel shuffling
	   NULL         - 0
	   RANDOM       - algorithm that ran was random ordering
	   RESULTS      - reference the results window object of an array
	   SPACE_DOMAIN - algorithm that ran was space domain correlation
	   TRUE         - turn an attribute on
	   XID_HANDLE   - handle to X ID value
	   XV_KEY_DATA  - data attached to an xview object
	   XV_SHOW      - map a window to the display (i.e. display window)
*/
void display_results()
{
	register char string[50];
	register GC gc;
	register Window xwin;

	if(info.alg != -1) {
	  if(isbitclear(BATCHMODE)) {
	    /*Get variables needed to draw text with Xlib*/
	    gc = (GC) xv_get(frame[RESULTS],XV_KEY_DATA,GC_HANDLE);
	    xwin = (Window) xv_get(frame[RESULTS],XV_KEY_DATA,XID_HANDLE);

	    XClearWindow(dpy,xwin);
	  }

	  /*Algorithm that ran*/
	  switch(info.alg) {
	    case SPACE_DOMAIN:
		sprintf(string,"   Test: Correlation (space domain)");
		break;
	    case FREQ_DOMAIN:
		sprintf(string,"   Test: Correlation (frequency domain)");
		break;
	    case LPS:
		sprintf(string,"   Test: Linear Pixel Shuffling");
		break;
	    case RANDOM:
		sprintf(string,"   Test: Random Ordering");
		break;
	  }
	  if(isbitclear(BATCHMODE))
	    XDrawString(dpy,xwin,gc,10,20,string,strlen(string));
	  else printf("%s\n",string);

	  /*Ratio between image and pattern*/
	  sprintf(string,"  Ratio: %.2lf:1 (image:pattern)",
		  (double) (header[IMAGE].pixels * header[IMAGE].scanlines) /
		  (double) (header[PATTERN].pixels *
			    header[PATTERN].scanlines));
	  if(isbitclear(BATCHMODE))
	    XDrawString(dpy,xwin,gc,10,40,string,strlen(string));
	  else printf("%s\n",string);

	  /*Time spent executing*/
          sprintf(string,"   Time: %02ld:%02ld:%02ld (h:m:s)",info.time.hours,
		  info.time.minutes,info.time.seconds);
	  if(isbitclear(BATCHMODE))
	    XDrawString(dpy,xwin,gc,10,60,string,strlen(string));
	  else printf("%s\n",string);

	  /*Pattern match location*/
	  sprintf(string,"    Pos: Match made at [%d,%d]",info.x,info.y);
	  if(isbitclear(BATCHMODE))
	    XDrawString(dpy,xwin,gc,10,80,string,strlen(string));
	  else printf("%s\n",string);

	  /*Image Name*/
	  sprintf(string,"  Image: %s",filename[IMAGE]);
	  if(isbitclear(BATCHMODE))
	    XDrawString(dpy,xwin,gc,10,100,string,strlen(string));
	  else printf("%s\n",string);

	  /*Pattern Name*/
	  if(filename[PATTERN])
	    sprintf(string,"Pattern: %s",filename[PATTERN]);
	  else
	    sprintf(string,"Pattern: from image");
	  if(isbitclear(BATCHMODE))
	    XDrawString(dpy,xwin,gc,10,120,string,strlen(string));
	  else printf("%s\n",string);

	  /*Special additional data depending on algorithm*/
	  if(info.alg == LPS) {
	    /*Maximum numbers used*/
	    sprintf(string,"  Parms: %ld and %ld",info.p1,info.p2);
	    if(isbitclear(BATCHMODE))
	      XDrawString(dpy,xwin,gc,10,140,string,strlen(string));
	    else printf("%s\n",string);

	    /*Increments (k1, k2) used*/
	    sprintf(string,"     Kn: %d and %d",info.k1,info.k2);
	    if(isbitclear(BATCHMODE))
	      XDrawString(dpy,xwin,gc,10,160,string,strlen(string));
	    else printf("%s\n",string);
	  } else if(info.alg == RANDOM) {
		   sprintf(string,"   Seed: %ld",info.seed);
		   if(isbitclear(BATCHMODE))
	             XDrawString(dpy,xwin,gc,10,140,string,strlen(string));
	           else printf("%s\n",string);
		 }

	  if(isbitclear(BATCHMODE))
	    xv_set(frame[RESULTS],XV_SHOW,TRUE,NULL);
	}
}

